@@ -2,6 +2,7 @@ from scipy import stats
 import shutil
 import importlib
 import xspec
+import astropy.io.fits as fits
 import os
 import numpy as np
 from IPython.display import Image
@@ -13,8 +14,15 @@ def view_group_spectra(r, reference_instrument, subcases_pattern, systematic_fra
 
     if reference_instrument.lower() == 'spi':
         ff=pf.open(r["reference_spec"],  mode='update')
-        ff[2].header['RESPFILE']=r["reference_rmf"]
-        reference_exposure=ff[2].header['EXPOSURE']
+        try:
+            reference_exposure=ff[2].header['EXPOSURE']
+            ff[2].header['RESPFILE']=r["reference_rmf"]
+        except:
+            try:
+                reference_exposure=ff[1].header['EXPOSURE']
+                ff[1].header['RESPFILE']=r["reference_rmf"]
+            except:
+                raise ValueError("Exposure should be in SPI spectrum!")
         reference_times='N/A'
         ff.flush()
         ff.close()
@@ -74,23 +82,31 @@ def basic_consistency(fit_by_lt, nh_sig_limit):
         d['nh_prob']=stats.chi2(d['ndof']).sf(d['chi2'])
         d['nh_sig']=stats.norm().isf(d['nh_prob'])
         print("%.1f\t%.2f\t%.2f\t%.2f"%(float(lt), d['chi2_red'], d['nh_prob'], d['nh_sig']) )
-        
-    good_lt = min([p for p in fit_by_lt.items() if p[1]['nh_sig']< nh_sig_limit])
+    try:
+        good_lt = min([p for p in fit_by_lt.items() if p[1]['nh_sig'] < nh_sig_limit])
+    except:
+        good_lt = None
     best_lt = min([p for p in fit_by_lt.items()], key=lambda x:x[1]['chi2_red'])
 
     if good_lt != best_lt:
         print("\033[31mNote that in this case, best LT does not coincde with the good LT. We want broader energy range\033[0m")
 
-    return good_lt
+    if good_lt is not None:
+        return good_lt
+    else:
+        return best_lt
 
 
-def fit(data, reference_instrument, model_setter, emin_values, fn_prefix=""):
+def fit(data, reference_instrument, model_setter, emin_values, fn_prefix="", systematic_fraction=0):
     importlib.reload(xspec)
 
+    if isinstance(emin_values, str):
+        emin_values = map(float, emin_values.split(","))
+
     fit_by_lt = {}
     fn_by_lt={}
 
-    xspec.AllModels.systematic=0.0
+    xspec.AllModels.systematic=systematic_fraction
 
     for c_emin in emin_values:
 
@@ -105,14 +121,17 @@ def fit(data, reference_instrument, model_setter, emin_values, fn_prefix=""):
         m1=xspec.AllModels(1)
 
         if ref_ind is not None:
-            xspec.Fit.error("1.0 max %.1f 1-%d"%(max_chi,len(ref_ind)*m1.nParameters))
-            ref=ref_ind[0]
+            n_spec = 2
+            if isinstance(ref_ind, list):
+                n_spec = len(ref_ind) + 1
+            xspec.Fit.error("1.0 max %.1f 1-%d"%(max_chi, n_spec*m1.nParameters))
+            ref = ref_ind[0]
         else:
-            xspec.Fit.error("1.0 max %.1f 1-%d"%(max_chi,m1.nParameters))
+            xspec.Fit.error("1.0 max %.1f 1-%d" % (max_chi,m1.nParameters))
 
         models={}
 
-        lt_key='%.10lg'%c_emin    
+        lt_key='%.10lg'%c_emin
     
         fit_by_lt[lt_key]=dict(
                 emin=c_emin,
@@ -122,7 +141,7 @@ def fit(data, reference_instrument, model_setter, emin_values, fn_prefix=""):
                 models=models,
             )
             
-        for m,ss in (isgri, 'isgri'), (ref, 'ref'):
+        for m, ss in (isgri, 'isgri'), (ref, 'ref'):
             if m is None: continue
 
             #initialize dictionaries
@@ -151,7 +170,7 @@ def fit(data, reference_instrument, model_setter, emin_values, fn_prefix=""):
         if os.path.exists(xcmfile):
             os.remove(xcmfile)
 
-        xspec.XspecSettings.save(xspec.XspecSettings,xcmfile, info='a')
+        xspec.XspecSettings.save(xspec.XspecSettings, xcmfile, info='a')
 
         xspec.Plot.device="/png"
         #xspec.Plot.addCommand("setplot en")
@@ -170,7 +189,7 @@ def fit(data, reference_instrument, model_setter, emin_values, fn_prefix=""):
     return fit_by_lt, fn_by_lt
 
 
-def parameter_comparison(good_lt, ng_sig_limit, reference_instrument, flux_tolerance=0.05):
+def parameter_comparison(good_lt, ng_sig_limit, reference_instrument, flux_tolerance=0.1):
     #compare parameters of best fit
     parameter_comparison={}
     best_result=good_lt[1]
@@ -180,45 +199,80 @@ def parameter_comparison(good_lt, ng_sig_limit, reference_instrument, flux_toler
         except:
             print("parameter %s is not in isgri parameter list"%(par_name))
             continue
-        isgri_value= par_values_isgri[0]
-        ref_value  = par_values_ref[0]
+        isgri_value = par_values_isgri[0]
+        ref_value = par_values_ref[0]
         
         if ref_value > isgri_value:
-            err_ref  =  ref_value - par_values_ref[1]
+            err_ref = ref_value - par_values_ref[1]
             err_isgri = par_values_isgri[2] - isgri_value
         else:
-            err_ref  =  - (ref_value - par_values_ref[2] )
-            err_isgri = - (par_values_isgri[1] - isgri_value )
+            err_ref = - (ref_value - par_values_ref[2])
+            err_isgri = - (par_values_isgri[1] - isgri_value)
 
-        par_diff_sigmas = (isgri_value - ref_value) /np.sqrt(err_ref**2 + err_isgri**2)
-        
-        print(par_name+" %.2f +/- %.2f ; %.2f +/- %.2f ; %.1f"%(isgri_value, err_isgri, ref_value, err_ref, par_diff_sigmas))
+        par_diff_sigmas = (isgri_value - ref_value) / np.sqrt(err_ref**2 + err_isgri**2)
+
+        print(par_name + " %.2f +/- %.2f ; %.2f +/- %.2f ; %.1f" % (
+        isgri_value, err_isgri, ref_value, err_ref, par_diff_sigmas))
         
         #We do not compare the normalization with NuSTAR strictly, because of non-simultaneity issues
-        if ( ('log10Flux' in par_name) or ('norm'in par_name) ) and (reference_instrument.lower() == 'nustar') :
-            success=True
-        elif 'log10Flux' in par_name:
+        if (('g10Flux' in par_name) or ('norm' in par_name)) and (reference_instrument.lower() == 'nustar') :
+            print("Not comparing %s for %s"%(par_name, reference_instrument))
+            success = True
+        elif 'g10Flux' in par_name and np.abs(par_diff_sigmas) > ng_sig_limit:
             frac_difference = np.abs( 1 - 10**(isgri_value - ref_value))
+            print("Using a fractional difference of %f to compare %s for %s" % (frac_difference,
+                                                                                par_name, reference_instrument))
             success = bool(frac_difference < flux_tolerance)
-        elif 'norm' in par_name:
-            frac_difference = np.abs( 1 - isgri_value /ref_value)
+        elif 'norm' in par_name and np.abs(par_diff_sigmas) > ng_sig_limit:
+            frac_difference = np.abs( 1 - isgri_value / ref_value)
+            print("Using a fractional difference of %f to compare %s for %s" % (frac_difference,
+                                                                                par_name, reference_instrument))
             success = bool(frac_difference < flux_tolerance)
         else:
+            #print("standard comparison for %s"%par_name)
             success = bool(np.abs(par_diff_sigmas) < ng_sig_limit)
-        
-        
+
         parameter_comparison[par_name] = dict(
-        
             ref = [ref_value, err_ref],
             isgri = [isgri_value, err_isgri ],
-            
             significance = np.abs(par_diff_sigmas),
-            
             success=success
-            
-            
-            
         )
 
     return parameter_comparison
 
+
+def write_down_oda_spectrum(spectrum, write_to_dir):
+    all_ext = []
+
+    files_by_source_name = {}
+
+    for name in set([s.meta_data['src_name'] for s in spectrum._p_list]):
+        print(f"writting down spectra for {name}")
+        
+        d = {}
+        files_by_source_name[name] = d
+        
+        src_name_stub = name.replace(' ', '_').replace('+','p')    
+        
+        d['spec_fn'] = write_to_dir + "/isgri_spectrum_{}.fits".format(src_name_stub)
+        d['arf_fn'] = write_to_dir + "/isgri_arf_{}.fits".format(src_name_stub)
+        d['rmf_fn'] = write_to_dir + "/isgri_rmf_{}.fits".format(src_name_stub)
+
+        specprod_for_source = [l for l in spectrum._p_list if l.meta_data['src_name'] == name ]
+
+        for p in specprod_for_source:
+            print(p.meta_data)
+
+        specprod_for_source[0].write_fits_file(d['spec_fn'])
+        specprod_for_source[1].write_fits_file(d['arf_fn'])
+        specprod_for_source[2].write_fits_file(d['rmf_fn'])
+        
+        all_ext.extend(fits.open(d['spec_fn'])[1:])
+        all_ext.extend(fits.open(d['arf_fn'])[1:])
+        all_ext.extend(fits.open(d['rmf_fn'])[1:])
+        
+    all_spectra_file = "isgri_all.fits"
+    fits.HDUList([fits.PrimaryHDU()] + all_ext).writeto(all_spectra_file, overwrite=True)
+
+    return files_by_source_name, all_spectra_file
\ No newline at end of file
